@@ -12,7 +12,7 @@ use File::Spec     ();
 use constant NO_CACHE => $ENV{MOJO_ASSETPACK_NO_CACHE} || 0;
 use constant DEBUG    => $ENV{MOJO_ASSETPACK_DEBUG}    || 0;
 
-our $VERSION = '0.50';
+our $VERSION = '0.51';
 
 my $MTIME = time;
 
@@ -75,14 +75,15 @@ sub register {
     return $app->log->debug("AssetPack: Helper $helper() is already registered.");
   }
 
+  $self->{assets}    = {};
+  $self->{processed} = {};
+
   $self->_app($app);
   $self->_ua->server->app($app);
   $self->minify($config->{minify} // $app->mode ne 'development');
   $self->out_dir($self->_build_out_dir($config, $app));
   $self->base_url($config->{base_url}) if $config->{base_url};
-
-  $self->{assets}    = {};
-  $self->{processed} = {};
+  $self->_reloader($app, $config->{reloader}) if $config->{reloader};
 
   if (NO_CACHE) {
     $app->log->info('AssetPack Will rebuild assets on each request in memory');
@@ -122,9 +123,10 @@ sub _assets_from_memory {
       my $c    = shift;
       my $path = $c->req->url->path;
 
-      return if $c->res->code;
+      return if $c->req->is_handshake or $c->res->code;
       return unless $path->[1] and 0 == index "$path", $self->base_url;
       return unless my $asset = $c->asset->_asset($path->[1]);
+      return if $asset->{internal};
       $c->res->headers->last_modified(Mojo::Date->new($MTIME))
         ->content_type($c->app->types->type($asset->url =~ /\.(\w+)$/ ? $1 : 'txt') || 'text/plain');
       $c->reply->asset($asset);
@@ -289,6 +291,24 @@ sub _process_many {
   map { my $name = _name($_); $self->_process("$name.$ext" => $_) } @files;
 }
 
+sub _reloader {
+  my ($self, $app, $config) = @_;
+  my $reloader = $self->_asset('reloader.js');
+
+  return if !$config->{enabled} and $app->mode ne 'development';
+
+  warn "[ASSETPACK] Adding reloader asset and route\n" if DEBUG;
+  $reloader->url('reloader.js')->{internal} = 1;
+  $self->{assets}{'reloader.js'} = [$reloader];
+  push @{$app->renderer->classes}, __PACKAGE__;
+  $app->routes->get('/packed/reloader')->to(template => 'packed/reloader', strategy => 'document', %$config);
+  $app->routes->websocket('/packed/reloader/ws')->to(
+    cb => sub {
+      shift->on(message => sub { shift->send('pong'); });
+    }
+  )->name('assetpack.ws');
+}
+
 # utils
 sub _ext { local $_ = File::Basename::basename($_[0]); /\.(\w+)$/ ? $1 : 'unknown'; }
 
@@ -309,7 +329,7 @@ Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass, javascrip
 
 =head1 VERSION
 
-0.50
+0.51
 
 =head1 SYNOPSIS
 
@@ -522,3 +542,37 @@ Per Edin - C<info@peredin.com>
 Viktor Turskyi
 
 =cut
+
+__DATA__
+@@ packed/reloader.js.ep
+;window.addEventListener('load', function(e) {
+  var xhr, socket, t, reloaded = 0;
+  var connect = function() {
+    socket = new WebSocket('<%= url_for('assetpack.ws')->userinfo(undef)->to_abs %>'.replace(/^http/, 'ws'));
+    socket.onopen = function(e) {
+      if (reloaded++) {
+        xhr = new XMLHttpRequest();
+        xhr.responseType = 'document';
+        xhr.open('GET', window.location.href);
+        xhr.onreadystatechange = function() {
+          if (xhr.readyState != 4) return;
+          if (window.console) console.log('[AssetPack] Replacing <head>...</head>');
+          document.head.innerHTML = this.responseXML.getElementsByTagName('head')[0].innerHTML;
+        };
+        xhr.send(null);
+      }
+      t = setInterval(function() { socket.send('ping'); }, 5000);
+    }
+    socket.onclose = function() {
+      if (t) clearTimeout(t);
+      if (window.console) console.log('[AssetPack] Reloading with strategy "<%= $strategy %>" (' + reloaded + ')');
+      if ('<%= $strategy %>' == 'document') {
+        return window.location = window.location.href;
+      }
+      else {
+        setTimeout(function() { connect() }, 500);
+      }
+    };
+  };
+  connect();
+});
